#!/bin/bash
set -e

# install arch in a vm with dual-uefi/bios and optional encryption
# v1, 2020-04-18, ed <irc.rizon.net>, MIT-Licensed
# https://ocv.me/dev/?install-arch-usb.sh
#
# you can dump the vm disk to a usb after install (portable arch),
# the usb will be bootable on both uefi and bios machines,
# may want to add linux-firmware to pacstrap for this
#
# can also be used as a regular vm but then encryption is snakeoil,
# just provide a blank fde password to disable it
#
# note that /boot is not encrypted or protected; easy to add some
#   gadget which captures your passphrase the next time you use it
#
# steps
#   boot a vm with the arch iso
#   when you're at the shell, run this script and switch focus to vm
#   the install-script is typed into the vm and executed automatically
#   after reboot, in xfce, run https://ocv.me/dev/?rice-xfce.sh
#
# based on
#   https://gist.github.com/elerch/678941eb670324ffc3f261eabba81310
#   https://superuser.com/questions/801515/is-a-hybrid-linux-usb-stick-for-uefi-legacy-bios-possible


# fallback values
vm_wt="spice display 0:0"
vm_wt="archlinux on QEMU/KVM"
vm_wt="arch-efi on QEMU/KVM"

# overwrite fallback values with defines in ./config
[ -e config ] && . config


# funfact, the stuff below is the precursor to https://github.com/9001/diodes/kx/
focused() {
	wt="$(xdotool getactivewindow getwindowname getwindowfocus getwindowname | uniq)"
	printf '%s\n' "$wt" | wc -l | grep -qE '^1$' || return 1
	[ "$wt" = "$vm_wt" ]
}

scr() {
	local ln=$(cat -n "$0" | awk '/^\s+[0-9]+\s+# begin install script/ {print $1}')
	awk "NR>$ln" "$0"
}

say() {
	focused
	xdotool type --clearmodifiers --args 1 --delay 7 "$*"
	# libvirt can do delay 2, virtualbox needs 7
}

sum=$(
	scr | sha1sum | cut -c-40
)

sleep 0.2
xdotool search --name "^$vm_wt$" windowactivate
sleep 0.2

say "echo "
scr | gzip -c | base64 | while IFS= read -r x
do
	say "$x"
done
say " | base64 -d | gzip -d > meme; chmod 755 meme; sha1sum meme; echo $sum"
focused
xdotool key Return
sleep 1
say "./meme"
exit 0







# begin install script
#!/bin/bash
set -ex

decrypt_time_in_sec=5
timedatectl set-ntp true

# cleanup any previous runs
umount /mnt/{boot,} || true
cryptsetup close rv || true

blkdiscard /dev/sda
partprobe /dev/sda

stty -echo; set +x
printf '\n  provide a blank fde password to disable encryption\n\n'
read -u1 -rp 'password(fde)> ' pw_fde ;echo
read -u1 -rp 'password(root)> ' pw_root ;echo
[ -z "$pw_fde" ] && do_crypt= || do_crypt=1
set -x; stty echo

for m in uib.no kernel.org leaseweb acc.umu.se lysator.liu.se
do grep -F $m /etc/pacman.d/mirrorlist || true; done >/x
[ -s /x ] && mv /x /etc/pacman.d/mirrorlist || rm /x
sed -ri 's/^#(Color)/\1/' /etc/pacman.conf
pacman -Sy

sgdisk -Z /dev/sda || sgdisk -Z /dev/sda
sgdisk -o /dev/sda
sgdisk -n 1:1M:127M /dev/sda
sgdisk -t 1:ef00 /dev/sda     # efi sys (fat32)
sgdisk -A 1:set:2 /dev/sda    # legacy boot flag
sgdisk -n 2:128M:-1M /dev/sda
sgdisk -t 2:8300 /dev/sda     # rootfs
partprobe /dev/sda

dc=/dev/sda2
[ $do_crypt ] || dr=$dc  # dev:root = dev:crypt
[ $do_crypt ] && {
	(set +x;printf %s "$pw_fde") | cryptsetup luksFormat -c aes-xts-essiv:sha256 -h sha512 -s 512 -y -i ${decrypt_time_in_sec}000 $dc -
	(set +x;printf %s "$pw_fde") | cryptsetup open --allow-discards $dc rv -
	dr=/dev/mapper/rv
}

mkfs.fat -F 32 -n ARCHBOOT /dev/sda1
mkfs.xfs -f -L archroot $dr

while [ ! -e /dev/disk/by-label/archroot ]
  do read -t0.1 || echo -ne .; done; echo

mount /dev/disk/by-label/archroot /mnt
mkdir /mnt/boot
mount /dev/disk/by-label/ARCHBOOT /mnt/boot

#export ftp_proxy=http://172.17.0.1:3128/
#export http_proxy=http://172.17.0.1:3128/
pacstrap /mnt base vim linux syslinux mkinitcpio cryptsetup xfsprogs dhcpcd
genfstab -U -p /mnt | tee -a /mnt/etc/fstab

pad=$(head -c72 /dev/zero | tr '\0' '-')
printf '\n%s\n' $pad "$(ls -al /dev/disk/by-uuid)" $pad "$(cat /mnt/etc/fstab)" $pad
(set +e;cd /dev/disk/by-uuid; for x in *; do grep "$x" /mnt/etc/fstab; done) | wc -l | grep -q 2

uc=$(blkid -o value -s UUID $dc)  # uuid:crypt
ur=$(blkid -o value -s UUID $dr)  # uuid:root
[ $do_crypt ] &&
	echo "rv UUID=$uc none luks" | tee -a /mnt/etc/crypttab

{ cat <<'EOF'
set -e
#export ftp_proxy=http://172.17.0.1:3128/
#export http_proxy=http://172.17.0.1:3128/

sed -ri 's/^#(Color)/\1/' /etc/pacman.conf
echo parch > /etc/hostname
(set +x;printf '%s\n' "root:pw_root") | chpasswd
localectl set-locale LANG=en_US.UTF-8 && echo en_US.UTF-8 UTF-8 > /etc/locale.gen && locale-gen
ln -sf /usr/share/zoneinfo/Etc/UTC /etc/localtime
echo 'blacklist pcspkr' > /etc/modprobe.d/nobeep.conf
systemctl enable dhcpcd

[ do_crypt ] && {
	# move block before autodetect + add encrypt
	#   HOOKS=(base udev autodetect modconf block filesystems keyboard fsck)
	#   HOOKS=(base udev encrypt block autodetect modconf filesystems keyboard fsck)
	sed -ri 's/^(HOOKS=.)(.* )?encrypt( |.$)/\1\2\3/' /etc/mkinitcpio.conf
	sed -ri 's/^(HOOKS=.)(.* )?autodetect( | .* )block( |.$)/\1\2encrypt block autodetect\3\4/' /etc/mkinitcpio.conf
	sed -ri 's/^(HOOKS=.*)  (.*)/\1 \2/' /etc/mkinitcpio.conf
	grep ^HOOKS= /etc/mkinitcpio.conf
}

sed -ri 's/^#(COMPRESSION="xz")/\1/; s/^(COMPRESSION_OPTIONS=\()/\1-7T0/' /etc/mkinitcpio.conf

mkinitcpio -p linux

##
## uefi

mkdir -p /boot/EFI/syslinux
cp -pr /usr/lib/syslinux/efi64/* /boot/EFI/syslinux/
cd /boot/EFI
mv syslinux boot
cd boot
mv syslinux.efi bootx64.efi
cp -pvr /boot/syslinux/syslinux.cfg{,.bak}

# /boot/syslinux/syslinux.conf
#   root=/dev/sda3 rw
#   root=/dev/mapper/rv cryptdevice=UUID=deadbeef:rv:allow-discards rw
[ do_crypt ] &&
	sed -ri 's@root=/dev/sda[0-9]@root=/dev/mapper/rv cryptdevice=UUID=cryptuuid:rv:allow-discards@' /boot/syslinux/syslinux.cfg ||
	sed -ri 's@root=/dev/sda[0-9]@root=UUID=cryptuuid@' /boot/syslinux/syslinux.cfg

sed -ri 's/(^TIMEOUT )[0-9]+/\110/' /boot/syslinux/syslinux.cfg

##
## bios

dd bs=440 count=1 conv=notrunc if=/usr/lib/syslinux/bios/gptmbr.bin of=/dev/sda
#extlinux --install /boot || true
#cp -pr /usr/lib/syslinux/bios/* /boot/syslinux/

##
## xfce

pkgs=(xorg xfce4 lightdm lightdm-gtk-greeter sudo exfat-utils vlc \
  mpv ffmpeg chromium firefox openssh ttf-dejavu zip unzip dosfstools \
  ncdu vim curl axel screen tmux iftop sox w3m feh scrot lm_sensors \
  imagemagick xdelta3 lame flac jp2a pngquant milkytracker hexedit \
  python3 geany xsel cowsay figlet htop rxvt-unicode irssi rtorrent \
  unrar p7zip nmap traceroute dosbox conky bmon ncdu filezilla mc pigz \
  lbzip2 pixz pv wireshark-qt pulseaudio rxvt-unicode ranger \
  pulseaudio-alsa pavucontrol networkmanager network-manager-applet \
  xfce4-goodies papirus-icon-theme adapta-gtk-theme)

pacman --noconfirm -Syu
pacman --noconfirm -S "${pkgs[@]}"
yes | pacman -Scc

useradd -m ed
(set +x;printf '%s\n' "ed:pw_root") | chpasswd
echo '%wheel ALL=(ALL) ALL' > /etc/sudoers.d/wheel
usermod -aG wheel ed

(cd ~ed && curl -LO https://ocv.me/dev/rice-xfce.sh && chmod 755 rice-xfce.sh && chown ed. rice-xfce.sh) || true

systemctl enable lightdm.service
systemctl enable NetworkManager.service

fstrim -v /

printf '\n  all good, pls reboot\n\n'

EOF
} |
sed -r "s/pw_root/$pw_root/; s/do_crypt/$do_crypt/; s/cryptuuid/$uc/" |
arch-chroot /mnt
